from typing import Dict, Any
import logging
from pyspark.sql.functions import col, when, lit
from prefect import task

from Enum.DataTypeEnum import DataTypeEnum
from Utils.SparkUtils import createSession
from Input.Source import Input

logger = logging.getLogger()


class ETLProcess:

    def __init__(self, config: Dict[str, Any]) -> None:
        self.conf = config
        self.identifier = self.conf["identifier"]
        self.session = None

        self.inputs = {}

    def init(self, spark_jars: Dict[str, str], mongodb_uri: str = "") -> None:
        self.session = createSession(self.identifier, spark_jars, mongodb_uri)

    @task
    def reader(self) -> None:
        try:
            inputs = self.conf["inputs"]
            input_type = inputs["type"]
            provider = inputs["params"]["provider"] if "provider" in inputs["params"].keys() else None
            for input_obj in inputs["data"]:
                identifier = input_obj["identifier"]
                params = {"identifier": identifier, "path": input_obj["path"], "type": input_obj["input_type"],
                          "separator": input_obj["separator"], "schema": input_obj["schema"]}
                current_input = Input(input_type, self.session, params, provider)
                current_input.get_data()
                self.inputs.update({identifier: current_input.data})
        except Exception as e:
            raise AssertionError(f"Error in function extrayendo data. Reader. {e}")

    @task
    def set_schema(self) -> None:
        try:
            inputs = self.conf["inputs"]
            for input_obj in inputs["data"]:
                identifier = input_obj["identifier"]
                schema = input_obj["schema"]
                input_schema = self.create_schema(schema)
                columns, schema = input_schema["columns"], input_schema["schema"]
                self.inputs[identifier] = self.inputs[identifier].select(*columns)
                for column, datatype in schema:
                    self.inputs[identifier] = self.inputs[identifier].withColumn(column, col(column).cast(datatype))
        except Exception as e:
            raise AssertionError(f"Error procesando información. Process. {e}")

    def create_schema(self, schema: Dict[str, str]) -> Dict[str, Any]:
        response = {}
        try:
            columns = list(schema.keys())
            structure = []
            for column in columns:
                field = (column, DataTypeEnum[schema[column]].value())
                structure.append(field)
            response.update({"columns": columns, "schema": structure})
        except Exception as e:
            logger.error(f"Error leyendo esquema para el insumo {self.identifier}. {e}")
        finally:
            return response

    @task
    def process_gross(self, identifier: str) -> bool:
        success = False
        try:
            self.inputs[identifier] = self.inputs[identifier].withColumn("AGENTE_COMISIONA", col("CONSULTOR_NK"))
            self.inputs[identifier] = self.inputs[identifier].withColumn('SEGMENTO',
                when((col('PLAN_NOMBRE').contains('Inter')) & (col('CLIENTE_NATURALEZA') == 'Persona Juridica'), 'B2B')
                .when(col('PLAN_NOMBRE').contains('Neg'), 'B2B')
                .when(col('SERVICIO').contains('Prepaid'), 'PREP').otherwise('B2C'))
            self.inputs[identifier] = self.inputs[identifier].withColumn("TIPO_CANAL", lit("DIRECT"))
            success = True
        except Exception as e:
            raise AssertionError(f"Error transformando archivo gross. {e}")
        finally:
            return success

    @task
    def process_teams(self, identifier: str) -> bool:
        success = False
        try:
            self.inputs[identifier] = self.inputs[identifier].withColumn('SEGMENTO',
                 when(col('CLIENTE_NATURALEZA') == 'Persona Juridica', 'B2B')
                .otherwise('B2C'))
            self.inputs[identifier] = self.inputs[identifier].withColumn("TIPO_CANAL", lit("DIRECT"))
            success = True
        except Exception as e:
            raise AssertionError(f"Error transformando archivo equipo. {e}")
        finally:
            return success

    @task
    def write(self, identifier: str, prev_status: bool = True) -> None:
        try:
            self.inputs[identifier].printSchema()
            self.inputs[identifier].write.format("com.mongodb.spark.sql.DefaultSource"). \
                                    option("collection", identifier).mode("append").save()
        except Exception as e:
            logger.error(f"Erro guardando resultados. {e}")
